%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright Hansen, Heaton, Li (2007)                                   
% Last revised May 2008
% This program simulates the posterior distribution for lambda(1) and lambda(0)
% in the system [c_t-c_(t-1) e_t-c_(t-1) d1_t-d1_(t-1) d5_t-d5_(t-1)] where
% dividends from both portfolio 1 and portfolio 5 are included in the VAR
% restrict d_t not Granger Cause Ct and et
% VAR system is estimated by equation-by-equation OLS regression
% noninformative prior distribution for variance and OLS coefficient implies that
% posterior distribution for inverse of variance (percision) is chi-square 
% posterior distribution for OLS coefficient is normal
% Result:
% Table 3: Posterior percentile of gamma1*iota1(pi)*400 last row
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
load longrun.mat;
logc = log(c);
loge = log(e);
logd = log(d);
gc = diff(logc);
gd = diff(logd);
logec = loge -logc;
L = 5; % number of lags
T = length(logec)-L;
n = 2;
n2 = 1;

%Step 1: Estimate VAR with cointegration by OLS, y = [gc c-e gd1 gd5]
y{1} = gc(L:end);
y{2} = logec(L+1:end);
X{1} = [];
for i = 1:L-1
    X{1} = [gc(i:end-(L-i),1) logec(i+1:end-(L-i),1) X{1}];
end;
X{1} = [X{1} logec(1:end-L,1)];
X{1} = [ones(T,1) X{1}];
X{2} = [X{1} y{1}];

uc  = [1 0 0 0 zeros(1,(L-1)*(n+2))];
ud = [[0 0 1 0;0 0 0 1] zeros(2,(L-1)*(n+2))];

p1 = 1; % index for the portfolio 1
p2 = 5; % index for the portfolio 5
y{3} = gd(L:end,p1);
y{4} = gd(L:end,p2);
X{3} = [];
for i = 1:L-1
    X{3} = [gc(i:end-(L-i),1) logec(i+1:end-(L-i),1) gd(i:end-(L-i),p1) gd(i:end-(L-i),p2) X{3}];
end;
X{3} = [X{3} logec(1:end-L,1) y{1} y{2}];
X{3} = [ones(T,1) X{3}];
X{4} = [X{3} y{3}];
for k = 1:4
    XX{k}  = inv(X{k}'*X{k});
    b{k}   = inv(X{k}'*X{k})*X{k}'*y{k};
    err{k} = y{k} - X{k}*b{k};
    rss(k) = err{k}'*err{k};
    sig(k) = sqrt(rss(k)/T);    
end;
    
% Step 2: Stack the system into VAR(L) in difference
constant1 = [b{1}(1) b{2}(1)+b{1}(1)*b{2}(end)]';
constant2 = b{3}(1)+ b{3}(end-1:end)'*constant1;
constant3 = b{4}(1)+ b{4}(end-2:end)'*[constant1;constant2];
constant = [constant1;constant2;constant3];
tempB = [b{1}(2:end) b{2}(2:end-1)+b{1}(2:end)*b{2}(end)]';
B1 = [];
for i = 1:L-1
    B1 = [B1 tempB(:,(i-1)*n+1:i*n) zeros(n,2)];
end
B1 = [B1 zeros(n,1) tempB(:,end) zeros(n,2)];
tempb = [b{3} b{4}(1:end-1)+b{4}(end)*b{3}];
tempB2 = [tempb(2:end-n-1,:)' zeros(2,1) tempb(end-n,:)' zeros(2)];
B2 = tempB2 + tempb(end-1:end,:)'*B1;
A = [B1;B2];
V1 = [sig(1) 0; sig(1)*b{2}(end) sig(2)];
V2 = [tempb(end-1:end,:)'*V1 [sig(3) 0; sig(3)*b{4}(end) sig(4)]];
V  = [V1 zeros(n,2); V2]; 

% Step 3: rearrange the systems to VAR(1)
G  = [A;[eye((L-1)*(n+2)) zeros((L-1)*(n+2),(n+2))]];
H  = [V; zeros((L-1)*(n+2), (n+2))];

% Step 4: Calculate gamma1 and iota1 (pi)
tempG  = inv(eye(size(G))-G);
gamma1 = uc*tempG*H; 
iota1  = ud*tempG*H; 
price  = gamma1*iota1'*400; 
    
    
% Monte Carlo Simulation
knum=100000;
simdata_price = zeros(knum,2); % price as gamma1*iota1'
    
ncheck = 1000;
ntest = 0; 

for k=1:knum
    for i = 1:4 % four equations
        scale = chi2rnd(T);
        newalpha = sqrt(scale/rss(i));
        mean = - newalpha*b{i};
        newbetasc = mvnrnd(mean, XX{i});
        newb{i} = - (newbetasc/newalpha)';
        newsig(i) = 1/newalpha;
    end
    tempB = [newb{1}(2:end) newb{2}(2:end-1)+newb{1}(2:end)*newb{2}(end)]';
    B1 = [];
    for i = 1:L-1
        B1 = [B1 tempB(:,(i-1)*n+1:i*n) zeros(n,2)];
    end
    B1 = [B1 zeros(n,1) tempB(:,end) zeros(n,2)];
    tempb = [newb{3} newb{4}(1:end-1)+b{4}(end)*b{3}];
    tempB2 = [tempb(2:end-n-1,:)' zeros(2,1) tempb(end-n,:)' zeros(2)];
    B2 = tempB2 + tempb(end-1:end,:)'*B1;
    newA = [B1;B2];
    V1 = [newsig(1) 0; newsig(1)*newb{2}(end) newsig(2)];
    V2 = [tempb(end-1:end,:)'*V1 [newsig(3) 0; newsig(3)*newb{4}(end) newsig(4)]];
    newV  = [V1 zeros(n,2); V2]; 
    newG  = [newA;[eye((L-1)*(n+2)) zeros((L-1)*(n+2),(n+2))]];
    newH  = [newV; zeros((L-1)*(n+2), (n+2))];
    
%    check = max(abs(eig(newG)));
%    if check < .9999
    if max(abs(eig(newG))) < 1
        ntest = ntest +1;
        tempG = inv(eye(size(newG))-newG);
        newgamma1 = uc*tempG*newH; 
        newiota1  = ud*tempG*newH;
        simdata_price(ntest,:) = newgamma1*newiota1'*400;            
        if  ntest > ncheck
            ntest
            ncheck = ncheck + 1000;
        end
    end
end
simdata_netprice = simdata_price(:,2)-simdata_price(:,1);

simdata_netprice_c = simdata_netprice(1:ntest);
prc = prctile(simdata_netprice_c,[5 25 50 75 95]);



